#include "song.h"
#include "utils.h"
#include "introduce_to_lcd.h"
#include "commands.h"

Ds1302 rtc(PIN_RST, PIN_SCLK, PIN_IO);

char subjects[6][6];
char delayed[6];
Ds1302::DateTime dts[6];

Ds1302::DateTime prev_dt = {
            .year = 100,
            .month = 13,
            .day = 32,
            .hour = 60,
            .minute = 60,
            .second = 60,
            .dow = 8
        };

LiquidCrystal_I2C lcd(LCD_ADDRESS, 16, 2);

int number_alarms = 0;

void setup() {
  
  lcd.init();
  lcd.clear();
  lcd.backlight();

  pinMode(BUTTON_DOWN, INPUT_PULLUP);
  pinMode(BUTTON_UP, INPUT_PULLUP);
  pinMode(BUTTON_MENU, INPUT_PULLUP);
  pinMode(BUTTON_NEXT, INPUT_PULLUP);
  pinMode(BUTTON_PREV, INPUT_PULLUP);
  pinMode(BUTTON_SELECT, INPUT_PULLUP);
  pinMode(BUZZER, OUTPUT);
  pinMode(PIR, INPUT);
  
  rtc.init();
  /* // Code if the RTC fails to print correct date time  
    Ds1302::DateTime dt;
    introduce_date_to_lcd(dt);
    rtc.setDateTime(&dt); */

  /* Initializeaza deadline-urilor si array-ul de delayed flag */
  for (int i = 0; i < 6; i++) {
    invalid_dt(&dts[i]);
    delayed[i] = NOT_NOW;
  }
}

void loop() {
  Ds1302::DateTime dt;
  int deadline_due = 0;

  /* Printeaza data si ora curenta extrasa din RTC */
  rtc.getDateTime(&dt);
  print_date(dt);

  /* Pastreaza o copie globala a datei afisate anterior pentru
   * afisarea eficienta a datei la lcd
   */
  copy_dt(&prev_dt, &dt);

  /* Itereaza prin toate deadline-urile pentru a verifica daca 
   * trebuie sunata alarma 
   */
  for (int i = 0; i < 6; i++) {

    if (is_invalid_dt(&dts[i])) {
      continue;
    }
    
    /* Extrage starea deadline-ului curent */
    int deadline_state = is_deadline_due(dt, dts[i]);

    /* Reface structura dt cu data curenta, aceasta fiind modificata anterior in
     * functia is_deadline_due 
     */
    copy_dt(&dt, &prev_dt);

    /* Citeste starea senzorului de miscare */
    int state_sensor = digitalRead(PIR);

    /* Daca senzorul detecteaza miscare */
    if (state_sensor == HIGH) {
          /* Daca starea deadline-ului indica faptul ca alarma trebuie activata */
          if (deadline_state != NOT_NOW){
            play_alarm(BUZZER, i, deadline_state);
            deadline_due = 1;
          /* Daca alarma a fost amanata anterior, acum trebuie activata */
          } else if (delayed[i] != NOT_NOW) {
            play_alarm(BUZZER, i, delayed[i]);
            deadline_due = 1;
          }

          /* Daca este ultima alarma a deadline-ului, atunci deadline-ul este sters*/
          if (deadline_state == NOW || delayed[i] == NOW) {
            invalid_dt(&dts[i]);
          }

          delayed[i] = NOT_NOW;
     /* Daca senzorul nu detecteaza miscare si starea deadline-ului este de a fi activata 
      * atunci se pastreaza in array-ul delayed pentru a putea fi sunata ulterior
      */
    } else if (deadline_state != NOT_NOW){
        delayed[i] = deadline_state;
    }

  }

  /* Curata LCD-ul daca alarma a fost activata */
  if (deadline_due == 1) {
    lcd.clear();
    invalid_dt(&prev_dt);
  }

  /* Daca functionalitatea meniului este activata prin buton, atunci 
   * se apeleaza functia display_menu
   */
  if (digitalRead(BUTTON_MENU) == LOW) {
    delay(200);
    display_menu();
    lcd.clear();
    invalid_dt(&prev_dt);
    delay(200);
  }
}
